@@ -23,6 +23,7 @@ import os
 import struct
 import sys
 import threading
+import re
 
 
 UINT64_BE = struct.Struct(">Q")
@@ -30,6 +31,8 @@ UINT32 = struct.Struct("<I")
 UINT32_BE = struct.Struct(">I")
 USHORT = struct.Struct("<H")
 
+TBPIECES = 7
+
 TRIANGLE = [
     6, 0, 1, 2, 2, 1, 0, 6,
     0, 7, 3, 4, 4, 3, 7, 0,
@@ -367,55 +370,30 @@ WDL_TO_DTZ = [-1, -101, 0, 101, 1]
 
 PCHR = ["K", "Q", "R", "B", "N", "P"]
 
-
-def filenames(one_king=True):
-    if not one_king:
-        for filename in all_dependencies(["PPPPPvP", "PPPPvPP", "PPPvPPP"], one_king=False):
-            yield filename
-        return
-
-    for i in range(1, 6):
-        yield "K%cvK" % (PCHR[i], )
-
-    for i in range(1, 6):
-        for j in range(i, 6):
-            yield "K%cvK%c" % (PCHR[i], PCHR[j])
-
-    for i in range(1, 6):
-        for j in range(i, 6):
-            yield "K%c%cvK" % (PCHR[i], PCHR[j])
-
-    for i in range(1, 6):
-        for j in range(i, 6):
-            for k in range(1, 6):
-                yield "K%c%cvK%c" % (PCHR[i], PCHR[j], PCHR[k])
-
-    for i in range(1, 6):
-        for j in range(i, 6):
-            for k in range(j, 6):
-                yield "K%c%c%cvK" % (PCHR[i], PCHR[j], PCHR[k])
-
-    for i in range(1, 6):
-        for j in range(i, 6):
-            for k in range(i, 6):
-                for l in range(j if i == k else k, 6):
-                    yield "K%c%cvK%c%c" % (PCHR[i], PCHR[j], PCHR[k], PCHR[l])
-
-    for i in range(1, 6):
-        for j in range(i, 6):
-            for k in range(j, 6):
-                for l in range(1, 6):
-                    yield "K%c%c%cvK%c" % (PCHR[i], PCHR[j], PCHR[k], PCHR[l])
-
-    for i in range(1, 6):
-        for j in range(i, 6):
-            for k in range(j, 6):
-                for l in range(k, 6):
-                    yield "K%c%c%c%cvK" % (PCHR[i], PCHR[j], PCHR[k], PCHR[l])
+TABLENAME_REGEX = re.compile(r"^[KQRBNP]+v[KQRBNP]+\Z")
 
 
-def normalize_filename(filename, mirror=False):
-    w, b = filename.split("v", 1)
+def is_table_name(name):
+    return len(name) <= 7 + 1 and TABLENAME_REGEX.match(name) and normalize_tablename(name) == name
+
+
+def tablenames(one_king=True, piece_count=6):
+    first = "K" if one_king else "P"
+
+    targets = []
+
+    white_pieces = piece_count - 2
+    black_pieces = 0
+    while white_pieces >= black_pieces:
+        targets.append(first + "P" * white_pieces + "v" + first + "P" * black_pieces)
+        white_pieces -= 1
+        black_pieces += 1
+
+    return all_dependencies(targets, one_king=one_king)
+
+
+def normalize_tablename(name, mirror=False):
+    w, b = name.split("v", 1)
     w = "".join(sorted(w, key=PCHR.index))
     b = "".join(sorted(b, key=PCHR.index))
     if mirror ^ ((len(w), [PCHR.index(c) for c in b]) < (len(b), [PCHR.index(c) for c in w])):
@@ -424,33 +402,32 @@ def normalize_filename(filename, mirror=False):
         return w + "v" + b
 
 
-def _dependencies(filename, one_king=True):
+def _dependencies(target, one_king=True):
+    w, b = target.split("v", 1)
+
     for p in PCHR:
         if p == "K" and one_king:
             continue
 
-        w, b = filename.split("v", 1)
-
         # Promotions.
         if p != "P" and "P" in w:
-            yield normalize_filename(w.replace("P", p, 1) + "v" + b)
+            yield normalize_tablename(w.replace("P", p, 1) + "v" + b)
         if p != "P" and "P" in b:
-            yield normalize_filename(w + "v" + b.replace("P", p, 1))
+            yield normalize_tablename(w + "v" + b.replace("P", p, 1))
 
         # Captures.
-        w, b = filename.split("v", 1)
         if p in w and len(w) > 1:
-            yield normalize_filename(w.replace(p, "", 1) + "v" + b)
+            yield normalize_tablename(w.replace(p, "", 1) + "v" + b)
         if p in b and len(b) > 1:
-            yield normalize_filename(w + "v" + b.replace(p, "", 1))
+            yield normalize_tablename(w + "v" + b.replace(p, "", 1))
 
 
-def dependencies(filename, one_king=True):
+def dependencies(target, one_king=True):
     closed = set()
     if one_king:
         closed.add("KvK")
 
-    for dependency in _dependencies(filename, one_king):
+    for dependency in _dependencies(target, one_king):
         if dependency not in closed and len(dependency) > 2:
             yield dependency
             closed.add(dependency)
@@ -461,17 +438,17 @@ def all_dependencies(targets, one_king=True):
     if one_king:
         closed.add("KvK")
 
-    open_list = [normalize_filename(filename) for filename in targets]
+    open_list = [normalize_tablename(target) for target in targets]
 
     while open_list:
-        filename = open_list.pop()
-        if filename in closed:
+        name = open_list.pop()
+        if name in closed:
             continue
 
-        yield filename
-        closed.add(filename)
+        yield name
+        closed.add(name)
 
-        open_list.extend(_dependencies(filename, one_king))
+        open_list.extend(_dependencies(name, one_king))
 
 
 def calc_key(board, mirror=False):
@@ -571,24 +548,26 @@ class PawnFileDataDtz(object):
 
 class Table(object):
 
-    def __init__(self, directory, filename, variant):
-        self.directory = directory
-        self.filename = filename
+    def __init__(self, path, variant=chess.Board):
+        self.path = path
         self.variant = variant
 
+        self.initialized = False
+        self.lock = threading.Lock()
         self.fd = None
         self.data = None
 
-        self.key = normalize_filename(filename)
-        self.mirrored_key = normalize_filename(filename, mirror=True)
+        tablename, _ = os.path.splitext(os.path.basename(path))
+        self.key = normalize_tablename(tablename)
+        self.mirrored_key = normalize_tablename(tablename, mirror=True)
         self.symmetric = self.key == self.mirrored_key
 
-        # Leave the v out of the filename to get the number of pieces.
-        self.num = len(filename) - 1
+        # Leave the v out of the tablename to get the number of pieces.
+        self.num = len(tablename) - 1
 
-        self.has_pawns = "P" in filename
+        self.has_pawns = "P" in tablename
 
-        black_part, white_part = filename.split("v")
+        black_part, white_part = tablename.split("v")
         if self.has_pawns:
             self.pawns = {}
             self.pawns[0] = white_part.count("P")
@@ -619,8 +598,7 @@ class Table(object):
     def init_mmap(self):
         # Open fd.
         if self.fd is None:
-            path = os.path.join(self.directory, self.filename) + self.suffix
-            self.fd = os.open(path, os.O_RDONLY | os.O_BINARY if hasattr(os, "O_BINARY") else os.O_RDONLY)
+            self.fd = os.open(self.path, os.O_RDONLY | os.O_BINARY if hasattr(os, "O_BINARY") else os.O_RDONLY)
 
         # Open mmap.
         if self.data is None:
@@ -1058,12 +1036,6 @@ class Table(object):
 
 class WdlTable(Table):
 
-    def __init__(self, directory, filename, variant=chess.Board, suffix=None):
-        super(WdlTable, self).__init__(directory, filename, variant)
-        self.suffix = suffix or variant.tbw_suffix
-        self.initialized = False
-        self.lock = threading.Lock()
-
     def init_table_wdl(self):
         with self.lock:
             self.init_mmap()
@@ -1083,8 +1055,8 @@ class WdlTable(Table):
             self.pieces = {}
 
             self.factor = {}
-            self.factor[0] = [0, 0, 0, 0, 0, 0]  # White
-            self.factor[1] = [0, 0, 0, 0, 0, 0]  # Black
+            self.factor[0] = [0 for _ in range(TBPIECES)]  # White
+            self.factor[1] = [0 for _ in range(TBPIECES)]  # Black
 
             self.norm = {}
             self.norm[0] = [0 for _ in range(self.num)]  # White
@@ -1184,7 +1156,7 @@ class WdlTable(Table):
         self.files[f].pieces[0] = [self.read_ubyte(p_data + i + j) & 0x0f for i in range(self.num)]
         self.files[f].norm[0] = [0 for _ in range(self.num)]
         self.set_norm_pawn(self.files[f].norm[0], self.files[f].pieces[0])
-        self.files[f].factor[0] = [0, 0, 0, 0, 0, 0]
+        self.files[f].factor[0] = [0 for _ in range(TBPIECES)]
         self.tb_size[p_tb_size] = self.calc_factors_pawn(self.files[f].factor[0], order, order2, self.files[f].norm[0], f)
 
         order = self.read_ubyte(p_data) >> 4
@@ -1192,7 +1164,7 @@ class WdlTable(Table):
         self.files[f].pieces[1] = [self.read_ubyte(p_data + i + j) >> 4 for i in range(self.num)]
         self.files[f].norm[1] = [0 for _ in range(self.num)]
         self.set_norm_pawn(self.files[f].norm[1], self.files[f].pieces[1])
-        self.files[f].factor[1] = [0, 0, 0, 0, 0, 0]
+        self.files[f].factor[1] = [0 for _ in range(TBPIECES)]
         self.tb_size[p_tb_size + 1] = self.calc_factors_pawn(self.files[f].factor[1], order, order2, self.files[f].norm[1], f)
 
     def setup_pieces_piece(self, p_data):
@@ -1225,7 +1197,7 @@ class WdlTable(Table):
             bside = 0
 
         if not self.has_pawns:
-            p = [0, 0, 0, 0, 0, 0]
+            p = [0 for _ in range(TBPIECES)]
             i = 0
             while i < self.num:
                 piece_type = self.pieces[bside][i] & 0x07
@@ -1239,7 +1211,7 @@ class WdlTable(Table):
             idx = self.encode_piece(self.norm[bside], p, self.factor[bside])
             res = self.decompress_pairs(self.precomp[bside], idx)
         else:
-            p = [0, 0, 0, 0, 0, 0]
+            p = [0 for _ in range(TBPIECES)]
             i = 0
             k = self.files[0].pieces[0][0] ^ cmirror
             color = k >> 3
@@ -1273,12 +1245,6 @@ class WdlTable(Table):
 
 class DtzTable(Table):
 
-    def __init__(self, directory, filename, variant=chess.Board, suffix=None):
-        super(DtzTable, self).__init__(directory, filename, variant)
-        self.suffix = suffix or variant.tbz_suffix
-        self.initialized = False
-        self.lock = threading.Lock()
-
     def init_table_dtz(self):
         with self.lock:
             self.init_mmap()
@@ -1290,7 +1256,7 @@ class DtzTable(Table):
                 not self.has_pawns and self.variant.pawnless_tbz_magic and
                 self.check_magic(self.variant.pawnless_tbz_magic))
 
-            self.factor = [0, 0, 0, 0, 0, 0]
+            self.factor = [0 for _ in range(TBPIECES)]
             self.norm = [0 for _ in range(self.num)]
             self.tb_size = [0, 0, 0, 0]
             self.size = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
@@ -1390,7 +1356,7 @@ class DtzTable(Table):
                 return 0, -1
 
             pc = self.pieces
-            p = [0, 0, 0, 0, 0, 0]
+            p = [0 for _ in range(TBPIECES)]
             i = 0
             while i < self.num:
                 piece_type = pc[i] & 0x07
@@ -1416,7 +1382,7 @@ class DtzTable(Table):
             bb = board.pieces_mask(piece_type, chess.WHITE if color == 0 else chess.BLACK)
 
             i = 0
-            p = [0, 0, 0, 0, 0, 0]
+            p = [0 for _ in range(TBPIECES)]
             for square in chess.scan_forward(bb):
                 p[i] = square ^ mirror
                 i += 1
@@ -1460,7 +1426,7 @@ class DtzTable(Table):
         self.files[f].norm = [0 for _ in range(self.num)]
         self.set_norm_pawn(self.files[f].norm, self.files[f].pieces)
 
-        self.files[f].factor = [0, 0, 0, 0, 0, 0]
+        self.files[f].factor = [0 for _ in range(TBPIECES)]
         self.tb_size[p_tb_size] = self.calc_factors_pawn(self.files[f].factor, order, order2, self.files[f].norm, f)
 
     def close(self):
@@ -1498,13 +1464,8 @@ class Tablebases(object):
             if len(self.lru) > self.max_fds:
                 self.lru.pop().close()
 
-    def _open_table(self, hashtable, directory, filename, Table, suffix, pawnless_suffix):
-        if os.path.isfile(os.path.join(directory, filename) + suffix):
-            table = Table(directory, filename, self.variant, suffix)
-        elif "P" not in filename and pawnless_suffix and os.path.isfile(os.path.join(directory, filename) + pawnless_suffix):
-            table = Table(directory, filename, self.variant, pawnless_suffix)
-        else:
-            return 0
+    def _open_table(self, hashtable, Table, path):
+        table = Table(path, self.variant)
 
         if table.key in hashtable:
             hashtable[table.key].close()
@@ -1521,19 +1482,27 @@ class Tablebases(object):
         (e.g. WDL files like ``KQvKN.rtbw`` and DTZ files like ``KRBvK.rtbz``)
         are loaded.
 
-        Returns the number of successfully openened and loaded tablebase files.
+        Returns the number of tablebases files that were found.
         """
         num = 0
         directory = os.path.abspath(directory)
 
-        if not os.path.isdir(directory):
-            raise IOError("not a directory: {0}".format(repr(directory)))
+        for filename in os.listdir(directory):
+            path = os.path.join(directory, filename)
+            tablename, ext = os.path.splitext(filename)
 
-        for filename in filenames(one_king=self.variant.one_king):
-            if load_wdl:
-                num += self._open_table(self.wdl, directory, filename, WdlTable, self.variant.tbw_suffix, self.variant.pawnless_tbw_suffix)
-            if load_dtz:
-                num += self._open_table(self.dtz, directory, filename, DtzTable, self.variant.tbz_suffix, self.variant.pawnless_tbz_suffix)
+            if is_table_name(tablename) and os.path.isfile(path):
+                if load_wdl:
+                    if ext == self.variant.tbw_suffix:
+                        num += self._open_table(self.wdl, WdlTable, path)
+                    elif "P" not in tablename and ext == self.variant.pawnless_tbw_suffix:
+                        num += self._open_table(self.wdl, WdlTable, path)
+
+                if load_dtz:
+                    if ext == self.variant.tbz_suffix:
+                        num += self._open_table(self.dtz, DtzTable, path)
+                    elif "P" not in tablename and ext == self.variant.pawnless_tbz_suffix:
+                        num += self._open_table(self.dtz, DtzTable, path)
 
         return num
 
@@ -1676,8 +1645,8 @@ class Tablebases(object):
             raise KeyError("syzygy tables do not contain positions with castling rights: {0}".format(board.fen()))
 
         # Validate piece count.
-        if chess.popcount(board.occupied) > 6:
-            raise KeyError("syzygy tables support up to 6 pieces, not {0}: {1}".format(chess.popcount(board.occupied), board.fen()))
+        if chess.popcount(board.occupied) > 7:
+            raise KeyError("syzygy tables support up to 6 (and experimentally 7) pieces, not {0}: {1}".format(chess.popcount(board.occupied), board.fen()))
 
         # Probe.
         v, _ = self.probe_ab(board, -2, 2)
